#!/usr/bin/env python3
import ctypes
from ctypes import *
from enum import Enum
from pydbus import SystemBus
import syslog

KLOG_ERROR   = 3
KLOG_WARNING = 4
KLOG_NOTICE  = 5
KLOG_INFO    = 6
KLOG_DEBUG   = 7

class KdkLogAuditType(Enum):
    KDK_LOG_AUDIT_TYPE_UNKNOWN = 0
    KDK_LOG_AUDIT_TYPE_CALL = 1
    KDK_LOG_AUDIT_TYPE_EXIT = 2

class KdkLogInterfaceType(Enum):
	KDK_LOG_INTERFACE_TYPE_C = 0
	KDK_LOG_INTERFACE_TYPE_DBUS = 1

class KdkLogModule(Enum):
    KDK_MODULE_ACCOUNT = 0                      #账户安全
    KDK_MODULE_APPLICATION = 1                  #应用安全
    KDK_MODULE_DATA = 2                         #数据安全
    KDK_MODULE_DESKTOP = 3                      #桌面管控
    KDK_MODULE_DEVICE = 4                       #设备安全
    KDK_MODULE_FILE = 5                         #文件安全（防篡改）
    KDK_MODULE_NETWORK = 6                      #网络安全
    KDK_MODULE_PROCESS = 7                      #进程安全
    KDK_MODULE_USERAUTH = 8                     #用户身份识别
    KDK_MODUlE_LOG = 9                          #安全SDK日志
    KDK_MODULE_SECDAEMON = 10                   #安全SDK初始化
    KDK_MODULE_APPLICATION_SYSSECRISKBOX = 11   #应用安全system风险提示服务
    KDK_MODULE_MAX = 12

TIME_SIZE = 100
MESG_SIZE = 1024
COMM_SIZE = 512
DBUS_SIZE = 256
class KdkLogMessage(Structure):
    _fields_ = [
        ('level', c_int),
        ('module', c_char * COMM_SIZE),
        ('pid', c_int),
        ('uid', c_int),
        ('path', c_char * COMM_SIZE),
        ('func', c_char * COMM_SIZE),
        ('time', c_char * TIME_SIZE),
        ('mesg', c_char * MESG_SIZE),
        ('audit_type', c_int),
        ('interface_type', c_int),
        ('dbus_name', c_char * DBUS_SIZE),
        ('dbus_path', c_char * DBUS_SIZE),
        ('dbus_interface', c_char * DBUS_SIZE)
    ]

class KdkSecLog:
    def __init__(self):
        try:
            self.liblogsec = CDLL("libkylogsec.so")
            self.dbus = SystemBus().get("org.freedesktop.DBus", "/org/freedesktop/DBus")
        except Exception:
            exit(-1)

    def _kdklog(self, level, module, pid, uid, audit_type, interface_type, dbus_name, dbus_path, dbus_interface, func, message):
        self.liblogsec.kdk_log_init.restype = POINTER(KdkLogMessage)
        klog_mesg = self.liblogsec.kdk_log_init(c_int(level), 
                                    create_string_buffer(module.encode()), 
                                    c_int(pid), 
                                    c_int(uid), 
                                    c_int(audit_type), 
                                    c_int(interface_type), 
                                    create_string_buffer(dbus_name.encode()), 
                                    create_string_buffer(dbus_path.encode()), 
                                    create_string_buffer(dbus_interface.encode()), 
                                    create_string_buffer(func.encode()))
        self.liblogsec.kdk_log_write(klog_mesg, create_string_buffer(message.encode()))

    def _get_log_module_name(self, module):
        if module == KdkLogModule.KDK_MODULE_ACCOUNT.value:
            module_name = "libkysdk-acctsec"
        elif module == KdkLogModule.KDK_MODULE_APPLICATION.value:
            module_name = "libkysdk-applicationsec"
        elif module == KdkLogModule.KDK_MODULE_DATA.value:
            module_name = "libkysdk-datasec"
        elif module == KdkLogModule.KDK_MODULE_DESKTOP.value:
            module_name = "libkysdk-desktopctrl"
        elif module == KdkLogModule.KDK_MODULE_DEVICE.value:
            module_name = "libkysdk-devicesec"
        elif module == KdkLogModule.KDK_MODULE_FILE.value:
            module_name = "libkysdk-filesec"
        elif module == KdkLogModule.KDK_MODULE_NETWORK.value:
            module_name = "libkysdk-networksec"
        elif module == KdkLogModule.KDK_MODULE_PROCESS.value:
            module_name = "libkysdk-processsec"
        elif module == KdkLogModule.KDK_MODULE_USERAUTH.value:
            module_name = "libkysdk-userauth"
        elif module == KdkLogModule.KDK_MODUlE_LOG.value:
            module_name = "libkysdk-logsec"
        elif module == KdkLogModule.KDK_MODULE_SECDAEMON.value:
            module_name = "kysdk-security-daemon"
        elif module == KdkLogModule.KDK_MODULE_APPLICATION_SYSSECRISKBOX.value:
            module_name = "libkysdk-applicationsec:system secriskbox"
        else:
            module_name = "unknown"
        return module_name

    def kdklog_dbus(self, sender, level, module, dbus_name, dbus_path, dbus_interface, func, message=None):
        if sender != None:
            pid = self.dbus.GetConnectionUnixProcessID(sender)
            uid = self.dbus.GetConnectionUnixUser(sender)
            module_name = self._get_log_module_name(module)
            self._kdklog(level, module_name, pid, uid, KdkLogAuditType.KDK_LOG_AUDIT_TYPE_UNKNOWN.value, 
                            KdkLogInterfaceType.KDK_LOG_INTERFACE_TYPE_DBUS.value, dbus_name, dbus_path, dbus_interface, func, message)

    def kdklog_dbus_call(self, sender, module, dbus_name, dbus_path, dbus_interface, func):
        if sender != None:
            pid = self.dbus.GetConnectionUnixProcessID(sender)
            uid = self.dbus.GetConnectionUnixUser(sender)
            module_name = self._get_log_module_name(module)
            self._kdklog(KLOG_INFO, module_name, pid, uid, KdkLogAuditType.KDK_LOG_AUDIT_TYPE_CALL.value, 
                            KdkLogInterfaceType.KDK_LOG_INTERFACE_TYPE_DBUS.value, dbus_name, dbus_path, dbus_interface, func, "")

    def kdklog_dbus_exit(self, sender, module, dbus_name, dbus_path, dbus_interface, func):
        if sender != None:
            pid = self.dbus.GetConnectionUnixProcessID(sender)
            uid = self.dbus.GetConnectionUnixUser(sender)
            module_name = self._get_log_module_name(module)
            self._kdklog(KLOG_INFO, module_name, pid, uid, KdkLogAuditType.KDK_LOG_AUDIT_TYPE_EXIT.value, 
                            KdkLogInterfaceType.KDK_LOG_INTERFACE_TYPE_DBUS.value, dbus_name, dbus_path, dbus_interface, func, "")

    def kdklog_dbus_call_with_params(self, sender, module, dbus_name, dbus_path, dbus_interface, func, params):
        if sender != None:
            pid = self.dbus.GetConnectionUnixProcessID(sender)
            uid = self.dbus.GetConnectionUnixUser(sender)
            module_name = self._get_log_module_name(module)
            self._kdklog(KLOG_INFO, module_name, pid, uid, KdkLogAuditType.KDK_LOG_AUDIT_TYPE_CALL.value, 
                            KdkLogInterfaceType.KDK_LOG_INTERFACE_TYPE_DBUS.value, dbus_name, dbus_path, dbus_interface, func, params)